@@ -28,6 +28,11 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	Wietse Venema
+/*	Google, Inc.
+/*	111 8th Avenue
+/*	New York, NY 10011, USA
 /*--*/
 
 /* System library. */
@@ -43,6 +48,14 @@
 /* Global library. */
 
 #include <mail_params.h>
+#include <debug_peer.h>
+
+ /*
+  * TLS library.
+  */
+#if defined(USE_TLS) && defined(USE_TLSRPT)
+#include <tlsrpt_wrapper.h>
+#endif
 
 /* Application-specific. */
 
@@ -55,14 +68,21 @@ SMTP_STATE *smtp_state_alloc(void)
 {
     SMTP_STATE *state = (SMTP_STATE *) mymalloc(sizeof(*state));
 
-    state->misc_flags = SMTP_MISC_FLAG_DEFAULT;
+    state->misc_flags = 0;
     state->src = 0;
     state->service = 0;
     state->request = 0;
     state->session = 0;
     state->status = 0;
     state->space_left = 0;
-    state->nexthop_domain = 0;
+    state->iterator->request_nexthop = vstring_alloc(100);
+    state->iterator->dest = vstring_alloc(100);
+    state->iterator->host = vstring_alloc(100);
+    state->iterator->addr = vstring_alloc(100);
+    state->iterator->saved_dest = vstring_alloc(100);
+#ifdef TLSRPT
+    state->tlsrpt = 0;
+#endif
     if (var_smtp_cache_conn) {
 	state->dest_label = vstring_alloc(10);
 	state->dest_prop = vstring_alloc(10);
@@ -76,25 +96,9 @@ SMTP_STATE *smtp_state_alloc(void)
 	state->endp_prop = 0;
 	state->cache_used = 0;
     }
-    state->dsn_reason = 0;
-
-    /*
-     * The process name, "smtp" or "lmtp", is also used as the DSN server
-     * reply type and for SASL service information lookup. Since all three
-     * external representations are identical there is no reason to transform
-     * from some external form X to some Postfix-specific canonical internal
-     * form, and then to transform from the internal form to external forms Y
-     * and Z.
-     */
-    if (strcmp(var_procname, "lmtp") == 0) {
-	state->misc_flags |= SMTP_MISC_FLAG_USE_LMTP;
-    } else if (strcmp(var_procname, "smtp") == 0) {
-	/* void */
-    } else {
-	msg_fatal("unexpected process name \"%s\" - "
-		  "specify \"smtp\" or \"lmtp\"",
-		  var_procname);
-    }
+    state->why = dsb_create();
+    state->debug_peer_per_nexthop = 0;
+    state->logged_line_length_limit = 0;
     return (state);
 }
 
@@ -102,6 +106,19 @@ SMTP_STATE *smtp_state_alloc(void)
 
 void    smtp_state_free(SMTP_STATE *state)
 {
+#ifdef USE_TLS
+    /* The TLS policy cache lifetime is one delivery. */
+    smtp_tls_policy_cache_flush();
+#endif
+    vstring_free(state->iterator->request_nexthop);
+    vstring_free(state->iterator->dest);
+    vstring_free(state->iterator->host);
+    vstring_free(state->iterator->addr);
+    vstring_free(state->iterator->saved_dest);
+#ifdef USE_TLSRPT
+    if (state->tlsrpt)
+	trw_free(state->tlsrpt);
+#endif
     if (state->dest_label)
 	vstring_free(state->dest_label);
     if (state->dest_prop)
@@ -111,9 +128,11 @@ void    smtp_state_free(SMTP_STATE *state)
     if (state->endp_prop)
 	vstring_free(state->endp_prop);
     if (state->cache_used)
-	htable_free(state->cache_used, (void (*) (char *)) 0);
-    if (state->dsn_reason)
-	vstring_free(state->dsn_reason);
+	htable_free(state->cache_used, (void (*) (void *)) 0);
+    if (state->why)
+	dsb_free(state->why);
+    if (state->debug_peer_per_nexthop)
+	debug_peer_restore();
 
-    myfree((char *) state);
+    myfree((void *) state);
 }
