@@ -80,8 +80,6 @@
 /*	New York, NY 10011, USA
 /*
 /*	Viktor Dukhovni
-/*
-/*	Wietse Venema
 /*--*/
 
 /* System library. */
@@ -104,6 +102,7 @@
 #include <msg.h>
 #include <mymalloc.h>
 #include <vstring.h>
+#include <sane_strtol.h>
 #include <stringops.h>
 #include <valid_hostname.h>
 #include <valid_utf8_hostname.h>
@@ -115,6 +114,10 @@
 #include <maps.h>
 #include <dsn_buf.h>
 
+/* TLS library. */
+
+#include <tlsrpt_wrapper.h>
+
 /* DNS library. */
 
 #include <dns.h>
@@ -223,15 +226,21 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
 {
     const char *lookup;
     char   *policy;
-    char   *saved_policy;
+    char   *saved_policy = 0;
     char   *tok;
-    const char *err;
     char   *name;
     char   *val;
     static VSTRING *cbuf;
+    char   *free_me = 0;
 
 #undef FREE_RETURN
-#define FREE_RETURN do { myfree(saved_policy); return; } while (0)
+#define FREE_RETURN do { \
+	if (saved_policy) \
+	    myfree(saved_policy); \
+	if (free_me) \
+	    myfree(free_me); \
+	return; \
+    } while (0)
 
 #define INVALID_RETURN(why, levelp) do { \
 	    MARK_INVALID((why), (levelp)); FREE_RETURN; } while (0)
@@ -252,7 +261,7 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
     }
     saved_policy = policy = mystrdup(lookup);
 
-    if ((tok = mystrtok_cw(&policy, CHARS_COMMA_SP, tls_policy->title)) == 0) {
+    if ((tok = mystrtokq(&policy, CHARS_COMMA_SP, CHARS_BRACE)) == 0) {
 	msg_warn("%s: invalid empty policy", WHERE);
 	INVALID_RETURN(tls->why, site_level);
     }
@@ -267,7 +276,7 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
      * Warn about ignored attributes when TLS is disabled.
      */
     if (*site_level < TLS_LEV_MAY) {
-	while ((tok = mystrtok_cw(&policy, CHARS_COMMA_SP, tls_policy->title)) != 0)
+	while ((tok = mystrtokq(&policy, CHARS_COMMA_SP, CHARS_BRACE)) != 0)
 	    msg_warn("%s: ignoring attribute \"%s\" with TLS disabled",
 		     WHERE, tok);
 	FREE_RETURN;
@@ -277,8 +286,12 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
      * Errors in attributes may have security consequences, don't ignore
      * errors that can degrade security.
      */
-    while ((tok = mystrtok_cw(&policy, CHARS_COMMA_SP, tls_policy->title)) != 0) {
-	if ((err = split_nameval(tok, &name, &val)) != 0) {
+    while ((tok = mystrtokq(&policy, CHARS_COMMA_SP, CHARS_BRACE)) != 0) {
+	const char *err;
+
+	if ((tok[0] == CHARS_BRACE[0]
+	     && (err = free_me = extpar(&tok, CHARS_BRACE, EXTPAR_FLAG_STRIP)) != 0)
+	    || (err = split_nameval(tok, &name, &val)) != 0) {
 	    msg_warn("%s: malformed attribute/value pair \"%s\": %s",
 		     WHERE, tok, err);
 	    INVALID_RETURN(tls->why, site_level);
@@ -336,9 +349,10 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
 		INVALID_RETURN(tls->why, site_level);
 		break;
 	    case TLS_LEV_FPRINT:
-		if (!tls->dane)
-		    tls->dane = tls_dane_alloc();
-		tls_dane_add_fpt_digests(tls->dane, val, "|", smtp_mode);
+		if (tls->matchargv == 0)
+		    tls->matchargv = argv_split(val, "|");
+		else
+		    argv_split_append(tls->matchargv, val, "|");
 		break;
 	    case TLS_LEV_VERIFY:
 	    case TLS_LEV_SECURE:
@@ -392,10 +406,110 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
 	    }
 	    continue;
 	}
+	/* Last one wins. */
+	if (!strcasecmp(name, "enable_rpk")) {
+	    /* Ultimately ignored at some security levels */
+	    if (strcasecmp(val, "yes") == 0) {
+		tls->enable_rpk = 1;
+	    } else if (strcasecmp(val, "no") == 0) {
+		tls->enable_rpk = 0;
+	    } else {
+		msg_warn("%s: attribute \"%s\" has bad value: \"%s\"",
+			 WHERE, name, val);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    continue;
+	}
+	/* Only one instance per policy. */
+	if (!strcasecmp(name, EXT_POLICY_TTL)) {
+	    char   *end;
+	    long    lval;
+
+	    if (tls->ext_policy_ttl != EXT_POLICY_TTL_UNSET) {
+		msg_warn("%s: attribute \"%s\" is specified multiple times",
+			 WHERE, name);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    if (!alldig(val) || ((lval = sane_strtol(val, &end, 10)),
+				 ((tls->ext_policy_ttl = lval) != lval))
+		|| *end != 0) {
+		msg_warn("%s: attribute \"%s\" has a malformed value: \"%s\"",
+			 WHERE, name, val);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    continue;
+	}
+	/* Only one instance per policy. */
+	if (!strcasecmp(name, EXT_POLICY_TYPE)) {
+	    if (tls->ext_policy_type) {
+		msg_warn("%s: attribute \"%s\" is specified multiple times",
+			 WHERE, name);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    if (!valid_tlsrpt_policy_type(val)) {
+		msg_warn("%s: attribute \"%s\" has an unexpected value: \"%s\"",
+			 WHERE, name, val);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    tls->ext_policy_type = mystrdup(val);
+	    continue;
+	}
+	if (!strcasecmp(name, EXT_POLICY_DOMAIN)) {
+	    if (tls->ext_policy_domain) {
+		msg_warn("%s: attribute \"%s\" is specified multiple times",
+			 WHERE, name);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    if (!valid_hostname(val, DO_GRIPE)) {
+		msg_warn("%s: attribute \"%s\" has a malformed value: \"%s\"",
+			 WHERE, name, val);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    tls->ext_policy_domain = mystrdup(val);
+	    continue;
+	}
+	/* Multiple instances per policy are allowed. */
+	if (!strcasecmp(name, EXT_POLICY_STRING)) {
+	    if (tls->ext_policy_strings == 0)
+		tls->ext_policy_strings = argv_alloc(1);
+	    argv_add(tls->ext_policy_strings, val, (char *) 0);
+	    continue;
+	}
+	/* Multiple instances per policy are allowed. */
+	if (!strcasecmp(name, EXT_MX_HOST_PATTERN)) {
+	    if (tls->ext_mx_host_patterns == 0)
+		tls->ext_mx_host_patterns = argv_alloc(1);
+	    argv_add(tls->ext_mx_host_patterns, val, (char *) 0);
+	    continue;
+	}
+	/* Only one instance per policy. */
+	if (!strcasecmp(name, EXT_POLICY_FAILURE)) {
+	    if (tls->ext_policy_failure != 0) {
+		msg_warn("%s: attribute \"%s\" is specified multiple times",
+			 WHERE, name);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    if (!valid_tlsrpt_policy_failure(val)) {
+		msg_warn("%s: attribute \"%s\" has an unexpected value: \"%s\"",
+			 WHERE, name, val);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    tls->ext_policy_failure = mystrdup(val);
+	    continue;
+	}
 	msg_warn("%s: invalid attribute name: \"%s\"", WHERE, name);
 	INVALID_RETURN(tls->why, site_level);
     }
-
+    if (tls->ext_policy_type == 0) {
+	if (tls->ext_policy_ttl || tls->ext_policy_strings
+	    || tls->ext_policy_domain || tls->ext_mx_host_patterns
+	    || tls->ext_policy_failure) {
+	    msg_warn("%s: built-in policy has unexpected attribute "
+		     "policy_ttl, policy_domain, policy_string, "
+		     "mx_host_pattern or policy_failure", WHERE);
+	    INVALID_RETURN(tls->why, site_level);
+	}
+    }
     FREE_RETURN;
 }
 
@@ -427,7 +541,7 @@ static void tls_policy_lookup(SMTP_TLS_POLICY *tls, int *site_level,
 
 /* load_tas - load one or more ta files */
 
-static int load_tas(TLS_DANE *dane, const char *files, const char *blame)
+static int load_tas(TLS_DANE *dane, const char *files)
 {
     int     ret = 0;
     char   *save = mystrdup(files);
@@ -435,7 +549,7 @@ static int load_tas(TLS_DANE *dane, const char *files, const char *blame)
     char   *file;
 
     do {
-	if ((file = mystrtok_cw(&buf, CHARS_COMMA_SP, blame)) != 0)
+	if ((file = mystrtok(&buf, CHARS_COMMA_SP)) != 0)
 	    ret = tls_dane_load_trustfile(dane, file);
     } while (file && ret);
 
@@ -520,6 +634,7 @@ static void *policy_create(const char *unused_key, void *context)
 
     smtp_tls_policy_init(tls, dsb_create());
     tls->conn_reuse = var_smtp_tls_conn_reuse;
+    tls->enable_rpk = var_smtp_tls_enable_rpk;
 
     /*
      * Compute the per-site TLS enforcement level. For compatibility with the
@@ -604,6 +719,13 @@ static void *policy_create(const char *unused_key, void *context)
      */
     set_cipher_grade(tls);
 
+/*
+ * Even when soliciting raw public keys, synthesize TLSA RRs that also match
+ * certificates.  Though this is fragile, it maintains compatibility with
+ * servers that never return RPKs.
+ */
+#define DONT_SUPPRESS_CERT_MATCH	0
+
     /*
      * Use main.cf cert_match setting if not set in per-destination table.
      */
@@ -619,32 +741,40 @@ static void *policy_create(const char *unused_key, void *context)
     case TLS_LEV_FPRINT:
 	if (tls->dane == 0)
 	    tls->dane = tls_dane_alloc();
-	if (tls->dane->tlsa == 0) {
-	    tls_dane_add_fpt_digests(tls->dane, var_smtp_tls_fpt_cmatch,
-				     CHARS_COMMA_SP, smtp_mode);
-	    if (tls->dane->tlsa == 0) {
-		msg_warn("nexthop domain %s: configured at fingerprint "
-		       "security level, but with no fingerprints to match.",
-			 dest);
-		MARK_INVALID(tls->why, &tls->level);
-		return ((void *) tls);
+	/* Process the specified fingerprint match patterns */
+	if (tls->matchargv) {
+	    int     i;
+
+	    for (i = 0; i < tls->matchargv->argc; ++i) {
+		tls_dane_add_fpt_digests(tls->dane, DONT_SUPPRESS_CERT_MATCH,
+					 tls->matchargv->argv[i], "",
+					 smtp_mode);
 	    }
+	} else {
+	    tls_dane_add_fpt_digests(tls->dane, DONT_SUPPRESS_CERT_MATCH,
+				     var_smtp_tls_fpt_cmatch, CHARS_COMMA_SP,
+				     smtp_mode);
+	}
+	if (tls->dane->tlsa == 0) {
+	    msg_warn("nexthop domain %s: configured at fingerprint "
+		     "security level, but with no fingerprints to match.",
+		     dest);
+	    MARK_INVALID(tls->why, &tls->level);
+	    return ((void *) tls);
 	}
 	break;
     case TLS_LEV_VERIFY:
     case TLS_LEV_SECURE:
 	if (tls->matchargv == 0)
 	    tls->matchargv =
-		argv_split_cw(tls->level == TLS_LEV_VERIFY ?
-			  var_smtp_tls_vfy_cmatch : var_smtp_tls_sec_cmatch,
-			  CHARS_COMMA_SP ":", tls->level == TLS_LEV_VERIFY ?
-			VAR_SMTP_TLS_VFY_CMATCH : VAR_SMTP_TLS_SEC_CMATCH);
+		argv_split(tls->level == TLS_LEV_VERIFY ?
+			   var_smtp_tls_vfy_cmatch : var_smtp_tls_sec_cmatch,
+			   CHARS_COMMA_SP ":");
 	if (*var_smtp_tls_tafile) {
 	    if (tls->dane == 0)
 		tls->dane = tls_dane_alloc();
 	    if (tls->dane->tlsa == 0
-		&& !load_tas(tls->dane, var_smtp_tls_tafile,
-			     VAR_SMTP_TLS_TAFILE)) {
+		&& !load_tas(tls->dane, var_smtp_tls_tafile)) {
 		MARK_INVALID(tls->why, &tls->level);
 		return ((void *) tls);
 	    }
@@ -679,6 +809,16 @@ static void policy_delete(void *item, void *unused_context)
     if (tls->dane)
 	tls_dane_free(tls->dane);
     dsb_free(tls->why);
+    if (tls->ext_policy_type)
+	myfree(tls->ext_policy_type);
+    if (tls->ext_policy_domain)
+	myfree(tls->ext_policy_domain);
+    if (tls->ext_policy_strings)
+	argv_free(tls->ext_policy_strings);
+    if (tls->ext_mx_host_patterns)
+	argv_free(tls->ext_mx_host_patterns);
+    if (tls->ext_policy_failure)
+	myfree(tls->ext_policy_failure);
 
     myfree((void *) tls);
 }
